C****************************** GAMMA **********************************
C*                                                                     *
C*    THIS PROGRAM PERFORMS ANALYSES OF FAILURE RATE ATTRIBUTE DATA.   *
C* TWO STATISTICAL MODELS MAY BE SPECIFIED: (1) A HOMOGENEOUS POISSON  *
C* MODEL IN WHICH THE FAILURE RATE IS ASSUMED EQUAL FOR ALL COMPONENTS,*
C* AND A COMPOUND POISSON-GAMMA MODEL IN WHICH THE FAILURE RATES FOR   *
C* EACH COMPONENT MAY VARY ACCORDING TO A GAMMA PRIOR DISTRIBUTION.    *
C*   FOR THE COMPOUND POISSON-GAMMA MODEL, THE PARAMETERS OF THE GAMMA *
C* PRIOR ARE ESTIMATED FROM THE GIVEN ATTRIBUTE DATA BY ANY OF THE     *
C* FOLLOWING THREE METHODS:                                            *
C*    (1) MATCHING UNWEIGHTED DATA MOMENTS TO THE PRIOR DISTRIBUTION   *
C*    (2) MATCHING UNWEIGHTED DATA MOMENTS TO THE MARGINAL DISTRIBUTION*
C*    (3) MAXIMUM LIKELIHOOD METHOD BASED ON THE MARGINAL DISTRIBUTION *
C* FOR EACH ESTIMATION TECHNIQUE, THE LIKELIHOOD OF THE SAMPLE CAN BE  *
C* REQUESTED, AND A CHI-SQUARED OR KOLMOGOROV-SMIRNOV GOODNESS-OF-FIT  *
C* TEST PERFORMED.                                                     *
C*                                                                     *
C*                                                                     *
C* INPUT DATA:                                                         *
C*                                                                     *
C*    CARD 1 (20A4)                                                    *
C*      TITLE(I) = ANY TITLE OR CAPTION TO IDENTIFY THE DATA (80 COL)  *
C*                                                                     *
C*    CARD 2 (4G20.5)                                                  *
C*      PCT      = FRACTION OF DISTRIBUTION AREA DESIRED IN THE UPPER  *
C*                 OR THE LOWER TAIL WHEN CALCULATING PERCENTILES OR   *
C*                 CONFIDENCE INTERVALS.                               *
C*      BMIN     = LOWER VALUE OF BETA FOR MAX. LIKELIHOOD SEARCH      *
C*                 (IF = 0.0 PROGRAM MAKES ESTIMATE OF LOWER BOUND)    *
C*      BMAX     = UPPER VALUE OF BETA FOR MAX. LIKELIHOOD SEARCH      *
C*                 (IF = 0.0 PROGRAM MAKES ESTIMATE FOR UPPER BOUND)   *
C*      EPS      = DESIRED ABSOLUTE ACCURACY FOR THE MAXIMUM LIKELI-   *
C*                 HOOD PARAMETER ESTIMATORS. (IF = 0.0 MAXIMUM LIKE-  *
C*                 LIHOOD METHOD IS OMITTED)                           *
C*                                                                     *
C*    CARD 3  (9I5)                                                    *
C*      NDATA    = NUMBER OF PAIRS OF FAILURE DATA (NF(I) AND NT(I))   *
C*      IHOMO    = INDEX CONTROLLING ANALYSIS BY HOMOGENEOUS MODEL:    *
C*                 = 1 IF ANALYSIS IS TO BE PREFORMED; =0 NO ANALYSIS  *
C*      IPMM     = INDEX FOR COMPOUND MODEL ANALYSIS WITH GAMMA PARA-  *
C*                 METERS ESTIMATED BY THE METHOD OF PRIOR MATCHING    *
C*                 MOMENTS: =1 FOR ANALYSIS; =0 NO SUCH ANALYSIS       *
C*      IMMM     = INDEX FOR COMPOUND MODEL ANALYSIS WITH GAMMA PARA-  *
C*                 METERS ESTIMATED BY THE METHOD OF MARGINAL MATCHING *
C*                 MOMENTS: =0 NO ANALYSIS; =1 PERFORM ANALYSIS        *
C*                 (NOTE: IF UNREALISTIC PARAMETER ESTIMATORS ARE OB-  *
C*                 TAINED, THE MAXIMUM LIKELIHOOD METHOD IS ATTEMPTED) *
C*      IML      = INDEX FOR COMPOUND MODEL ANALYSIS WITH GAMMA PARA-  *
C*                 METERS ESTIMATED BY THE METHOD OF MARGINAL MAXIMUM  *
C*                 LIKELIHOOD: =1 FOR THIS ANALYSIS; =0 FOR NO ANALYSIS*
C*                 COMPONENT: =0 NO ANALYSIS, =1 PERFORM ANALYSIS      *
C*      ILKHD    = SAMPLE LIKELIHOOD INDEX: =0 NOT CALCULATED, =1      *
C*                 CALCULATED                                          *
C*      IPOST    = INDEX FOR PERFORMING A POSTERIOR ANALYSIS FOR EACH  *
C*                 COMPONENT: =0 NO ANALYSIS; =1 PERFORM ANALYSIS      *
C*      ICHISQ   = PARAMETER CONTROLLING THE CHI-SQUARED GOODNESS-OF-  *
C*                 FIT TEST: =0 NO TEST; =1 PERFORM TEST; =-1 PERFORM  *
C*                 TEST AND GIVE INTERMEDIATE OUTPUT                   *
C*      IKSFIT   = PARAMETER CONTROLLING THE KOLMOGOROV-SMIRNOV        *
C*                 GOODNESS-OF-FIT TEST:  =0 NO TEST; =1 PERFORM TEST; *
C*                 =-1 PERFORM TEST AND GIVE INTERMEDIATE OUTPUT       *
C*                 (NOTE: TEST PERFORMED ONLY IF NUMBER OF COMPONENTS  *
C*                 IS LESS THAN 51)                                    *
C*                                                                     *
C*    CARD 4 .... (A8,3I10)  --  ONE CARD FOR EACH COMPONENT           *
C*      ID(I)    = IDENTIFICATION LABEL FOR I-TH COMPONENT (8 CHAR)    *
C*      NF(I)    = NUMBER OF OBSERVED FAILURES FOR I-TH COMPONENT      *
C*      NT(I)    = TOTAL TEST TIME FOR THE I-TH COMPONENT              *
C*      NN(I)    = NUMBER OF COMPONENTS WITH NF(I) FAILURES IN TIME    *
C*                 NT(I); IF OMITTED NN(I) ASSUMED = 1                 *
C*                                                                     *
C*                                                                     *
C* SUBPROGRAMS NEEDED:                                                 *
C*    VARS, POSTGM, GMPCTL, FGAM, MLGAMP, FN, DRTMI, LKLIHD, CHISQ,    *
C*    KSFIT, DKSLVL, KSSUM, H, DXP, GAMIC                              *
C*                                                                     *
C*                                                                     *
C* WRITTEN BY J.K. SHULTIS, KANSAS STATE UNIVERSITY  6/80.             *
C***********************************************************************
C
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 LAM(100),LAML(100),LAMU(100)
      REAL*8 TITLE(10),ID(100),LAMBAR,LAMLOW,LAMUP,LNPROB,VAR(4)
      COMMON /CLASS/LAM,LAML,LAMU
      COMMON /DTA/ID,NF(100),NT(100),NN(100),N,NDATA,IPRT,NITER
      COMMON/MODEL/AAA,BBB,II
      EXTERNAL FN,H
C
C
C  SET PRINTER UNIT=6; SET READER UNIT=5; REL. ACC OF PERCENTILES
C  IS SET BY EPPS; MAX NUMBER OF ITERATIONS FOR EVALUATING PERCENTILE
C  SET BY VARIABLE NITER
      OPEN(UNIT=5,FILE='GAMMA.INP',STATUS='OLD')
      OPEN(UNIT=6,FILE='GAMMA.OUT',STATUS='UNKNOWN')
      IPRT = 6
      IRDR = 5
      EPPS = 0.00000001
      NITER = 200
C  SET PARAMETERS FOR THE CHI-SQUARED FIT TEST
      EXPT = 3.0
      MAXBIN = 16
      LDF = 1
C
C
C  READ COMPONENT FAILURE DATA
    1 READ (IRDR,38,END=19) (TITLE(I),I=1,10)
      READ (IRDR,39) PCT,BMIN,BMAX,EPS
      WRITE (IPRT,40) (TITLE(I),I=1,10)
      READ (IRDR,41,END=19,ERR=19) NDATA,IHOMO,IPMM,IMMM,IML,ILKHD,IPOST
     1,ICHISQ,IKSFIT,(ID(I),NF(I),NT(I),NN(I),I=1,NDATA)
C  CALCULATE TOTAL NUMBER OF COMPONENTS N
      N = 0
      DO 2 I=1,NDATA
      IF (NN(I).LE.0) NN(I) = 1
    2 N = N+NN(I)
      WRITE (IPRT,42) (ID(I),NN(I),NF(I),NT(I),I=1,NDATA)
C  PRINT OUT THE INPUT OPTIONS
      WRITE (IPRT,27)
      IF (IHOMO.NE.0) WRITE (IPRT,20)
      IF (IPMM.NE.0) WRITE (IPRT,21)
      IF (IMMM.NE.0) WRITE (IPRT,22)
      IF (IML.NE.0) WRITE (IPRT,23)
      IF (ILKHD.EQ.1) WRITE (IPRT,28)
      IF (ICHISQ.NE.0) WRITE (IPRT,29)
      IF (IKSFIT.NE.0) WRITE (IPRT,30)
      IF (IPOST.EQ.1) WRITE (IPRT,31)
C
C  CALCULATE MEAN AND VARIANCE OF THE DATA
      SUMT = 0.0
      SUMF = 0.0
      SUMFT = 0.0
      SUMFT2 = 0.0
      SUMTI = 0.0
      DO 3 I=1,NDATA
      TT = NT(I)
      SUMT = SUMT+TT*NN(I)
      SUMTI = SUMTI+NN(I)/TT
      SUMF = SUMF+NF(I)*NN(I)
      AA = NF(I)/TT
      SUMFT = SUMFT+AA*NN(I)
    3 SUMFT2 = SUMFT2+AA*NN(I)*AA
      UMEAN = SUMFT/N
      UVAR = (SUMFT2-UMEAN*UMEAN*N)/(N-1)
C
C
C  CALCULATE HOMOGENEOUS PARAMETER ESTIMATE AND CONFIDENCE LIMITS
      LAMBAR = SUMF/SUMT
      HVAR = LAMBAR/SUMT
      STD = DSQRT(HVAR)
      PCCT = 1.0-PCT
      IF (IHOMO.EQ.0) GO TO 5
      CALL GMPCTL (LAMUP,LAMLOW,PCT,SUMF,SUMT,EPPS,1)
C     PRINT OUT RESULTS
      WRITE (IPRT,32)
      WRITE (IPRT,33)
      WRITE (IPRT,43) LAMBAR,STD,HVAR
      WRITE (IPRT,44) PCT,LAMLOW,PCCT,LAMUP
C   PERFORM GOODNESS OF FIT TEST
      ISEND = 1
      AAA = LAMBAR
      TTT = STD
      GO TO 15
    4 IF ((IPMM+IMMM+IML).EQ.0) GO TO 1
C
C  CALCULATE CLASSICAL FAILURE RATE AND ASSOC. CONFID. INT. FOR EACH
C  COMPONENT IN SAMPLE IF A POSTERIOR ANALYSIS IS REQUESTED (IPOST=1)
    5 IF (IPOST.EQ.0) GO TO 7
      DO 6 I=1,NDATA
      FF = NF(I)
      TT = NT(I)
      LAM(I) = FF/TT
      CALL GMPCTL (LAMUP,LAMLOW,PCT,FF,TT,EPPS,1)
      LAMU(I) = LAMUP
    6 LAML(I) = LAMLOW
C
C
C
C***   BEGIN ANALYSIS FOR COMPOUND MODEL;
C
C  CALCULATE MATCHING MOMENTS ESTIMATORS
    7 WRITE (IPRT,34)
      WRITE (IPRT,35)
      BETAUP = UVAR/UMEAN
      ALPHUP = UMEAN/BETAUP
      BETAUM = (UVAR-UMEAN*SUMTI/N)/UMEAN
      ALPHUM = UMEAN/BETAUM
      WRITE (IPRT,45)
C
C***  UNWEIGHTED MATCHING MOMENTS ESTIMATORS TO THE PRIOR - METHOD 1
      IF (IPMM.EQ.0) GO TO 9
      WRITE (IPRT,46)
      AAA = ALPHUP
      BBB = BETAUP
      TTT = 1./BBB
      AV = AAA*BBB
      VR = AV*BBB
      STD = DSQRT(VR)
      CALL GMPCTL (LAMUP,LAMLOW,PCT,AAA,TTT,EPPS,0)
      WRITE (IPRT,47) AAA,BBB,TTT
      CALL VARS (AAA,BBB,0,VAR,IVAR)
      IF (IVAR.EQ.0) WRITE (IPRT,24) (VAR(I),I=1,4)
      WRITE (IPRT,25) AV,STD,VR
      WRITE (IPRT,48) PCT,LAMLOW,PCCT,LAMUP
C   PERFORM GOODNESS OF FIT TESTS
      ISEND = 2
      GO TO 15
    8 IF (IPOST.EQ.1) CALL POSTGM (ALPHUP,BETAUP,PCT,EPPS)
C
C***  UNWEIGHTED MATCHING MOMENTS TO THE MARGINAL - METHOD 2
    9 IF (IMMM.EQ.0) GO TO 12
      WRITE (IPRT,49)
      IF ((ALPHUM.GT.0.0).AND.(BETAUM.GT.0.0)) GO TO 10
      WRITE (IPRT,51) ALPHUM,BETAUM
      IML = 1
      GO TO 12
   10 AAA = ALPHUM
      BBB = BETAUM
      TTT = 1./BBB
      AV = AAA*BBB
      VR = AV*BBB
      STD = DSQRT(VR)
      CALL GMPCTL (LAMUP,LAMLOW,PCT,AAA,TTT,EPPS,0)
      WRITE (IPRT,47) AAA,BBB,TTT
      CALL VARS (AAA,BBB,1,VAR,IVAR)
      IF (IVAR.EQ.0) WRITE (IPRT,24) (VAR(I),I=1,4)
      WRITE (IPRT,25) AV,STD,VR
      WRITE (IPRT,48) PCT,LAMLOW,PCCT,LAMUP
C   PERFORM GOODNESS OF FIT TEST
      ISEND = 3
      GO TO 15
   11 IF (IPOST.EQ.1) CALL POSTGM (ALPHUM,BETAUM,PCT,EPPS)
C
C***  MAXIMUM LIKELIHOOD ESTIMATORS - METHOD 3
   12 IF ((EPS.EQ.0.0).OR.(IML.EQ.0)) GO TO 1
      WRITE (IPRT,50)
      IF (BMIN.EQ.0.0) BMIN = BETAUP*0.01
      IF (BMAX.EQ.0.0) BMAX = 100.0*BETAUP
      CALL DRTMI (BETA,FF,FN,BMIN,BMAX,EPS,NITER,IER)
      IF (IER.NE.0) GO TO 14
      CALL MLGAMP (FFF,ALPHA,BETA)
      AAA = ALPHA
      BBB = BETA
      TTT = 1./BBB
      AV = AAA*BBB
      VR = AV*BBB
      STD = DSQRT(VR)
      CALL GMPCTL (LAMUP,LAMLOW,PCT,AAA,TTT,EPPS,0)
      WRITE (IPRT,47) AAA,BBB,TTT
      CALL VARS (AAA,BBB,2,VAR,IVAR)
      IF (IVAR.EQ.0) WRITE (IPRT,24) (VAR(I),I=1,4)
      WRITE (IPRT,25) AV,STD,VR
      WRITE (IPRT,48) PCT,LAMLOW,PCCT,LAMUP
C   PERFORM GOODNESS OF FIT TESTS
      ISEND = 4
      GO TO 15
   13 IF (IPOST.EQ.1) CALL POSTGM (ALPHA,BETA,PCT,EPPS)
      GO TO 1
   14 WRITE (IPRT,52) BMIN,BMAX
      GO TO 1
C
C
C  CALCULATE THE LIKELIHOOD PROBABILITY, CHI-SQUARED FIT, AND THE
C  KOLMORGOROV-SMIRNOV FIT
   15 IF (ILKHD.EQ.0) GO TO 16
      II = 1
      IF (ISEND.EQ.1) II = 0
      CALL LKLIHD (PROB,LNPROB,AAA,TTT,II)
      WRITE (IPRT,36) PROB,LNPROB
C  PERFORM CHI-SQUARED TEST OF FIT
   16 IF (ICHISQ.EQ.0) GO TO 17
      INTER = 0
      WRITE (IPRT,37)
      IF (ICHISQ.LT.0) INTER = 1
      CALL CHISQ (H,EXPT,MAXBIN,LDF,INTER,1.D-4,CHI2,NDF,SIG,NZ)
C  PERFORM K-S TEST OF FIT
   17 IF (IKSFIT.EQ.0) GO TO 18
      IOUT = 0
      IF (IKSFIT.LT.0) IOUT = 1
      WRITE (IPRT,26)
      CALL KSFIT (H,DSTAT,IOUT,SIG1,SIG2)
   18 GO TO (4,8,11,13), ISEND
C
C
   19 WRITE (IPRT,53)
      STOP
   20 FORMAT (59H ANALYSIS OF DATA WITH HOMOGENEOUS MODEL IS TO BE PERFO
     1RMED)
   21 FORMAT (100H ANALYSIS OF DATA WITH COMPOUND MODEL -- PRIOR PARAMET
     1ERS ESTIMATED BY PRIOR MATCHING MOMENTS METHOD)
   22 FORMAT (107H ANALYSIS OF DATA WITH COMPOUND MODEL -- PRIOR PARAMET
     1ERS ESTIMATED BY THE MARGINAL MATCHING MOMENTS METHOD)
   23 FORMAT (109H ANALYSIS OF DATA WITH COMPOUND MODEL -- PRIOR PARAMET
     1ERS ESTIMATED BY THE MARGINAL MAXIMUM LIKELIHOOD METHOD)
   24 FORMAT (21X,41HVARIANCE OF THE ESTIMATORS:   VAR(ALPHA)=,G11.4,13H
     1   VAR(BETA)=,G11.4/51X,18HCOVAR(ALPHA,BETA)=,G11.4,18H   CORRELAT
     2ION R =,G11.4)
   25 FORMAT (21X,36HMOMENTS OF ESTIMATED PRIOR:   MEAN =,G11.4,23H    S
     1TANDARD DEVIATION=,G11.4,12H  (VARIANCE=,G11.4,1H))
   26 FORMAT (////50H KOLMOGOROV-SMIRNOV DISCRETE GOODNESS-OF-FIT TEST:)
   27 FORMAT (///40H ********** OPTIONS REQUESTED **********/)
   28 FORMAT (38H SAMPLE LIKELIHOOD IS TO BE CALCULATED)
   29 FORMAT (49H CHI-SQUARED GOODNESS-OF-FIT TEST TO BE PERFORMED)
   30 FORMAT (65H KOLMOGOROV-SMIRNOV DISCRETE GOODNESS-OF-FIT TEST TO BE
     1 PERFORMED)
   31 FORMAT (80H AN ANALYSIS OF THE POSTERIOR DISTRIBUTION FOR EACH COM
     1PONENT IS TO BE PERFORMED)
   32 FORMAT (1H1,40X,45(1H*),/41X,1H*,43X,1H*,/41X,45H*  ANALYSIS FOR H
     1OMOGENEOUS POISSON MODEL:  *)
   33 FORMAT (41X,45H* ALL COMPONENTS HAVE THE SAME FAILURE RATE *,/41X,
     11H*,43X,1H*,/41X,45(1H*))
   34 FORMAT (1H1,40X,47(1H*),/41X,1H*,45X,1H*,/41X,1H*,8X,29HCOMPOUND P
     1OISSON-GAMMA MODEL:,8X,1H*)
   35 FORMAT (41X,47H* THE FAILURE RATES ARE DISTRIBUTED ACCORDING *,/41
     1X,1H*,11X,23HTO A GAMMA DISTRIBUTION,11X,1H*,/41X,1H*,45X,1H*,/41X
     2,47(1H*))
   36 FORMAT (1H ,////64H SAMPLE LIKELIHOOD (I.E. PROBABILITY OF OBSERVI
     1NG  GIVEN DATA) =,G15.6/10X,40HLOGARITHM (NATURAL BASE) OF LIKELIH
     2OOD =,G15.6)
   37 FORMAT (////34H CHI-SQUARED GOODNESS-OF-FIT TEST:)
   38 FORMAT (10A8)
   39 FORMAT (4G20.5)
   40 FORMAT (1H1,23(1H*),2X,10A8,2X,23(1H*))
   41 FORMAT (9I5/(A8,3I10))
   42 FORMAT (1H ,/54H NUMBER OF FAILURES AND TEST TIMES FOR EACH COMPON
     1ENT:,/1H ,4(3X,2HID,4X,9HNUM  F(I),4X,4HT(I),6X),/(1H ,4(A8,I4,I5,
     2I10,5X)))
   43 FORMAT (///25H ESTIMATED FAILURE RATE =,G11.4,38H   WITH ESTIMATED
     1 STANDARD DEVIATION =,G11.4,14H   (VARIANCE =,G11.4,1H))
   44 FORMAT (12X,26HCONFIDENCE BOUNDS: LAMBDA(,F6.4,9H LEVEL) =,G11.4,1
     11H    LAMBDA(,F6.4,9H LEVEL) =,G11.4)
   45 FORMAT (//132H ********** NON-HOMOGENEOUS MODEL - COMPONENTS HAVE
     1DIFFERENT FAILURE RATES. THE PRIOR DISTRIBUTION FOR THE FAILURE RA
     2TES **********,/15X,102H (LAMBDA) IS ASSUMED TO BE A GAMMA FUNCTIO
     3N WHOSE PARAMETERS ARE ESTIMATED FROM THE GIVEN FAILURE DATA)
   46 FORMAT (///97H *** METHOD 1 *** -- MATCHING THE FIRST TWO DATA MOM
     1ENTS TO THOSE OF THE PRIOR GAMMA DISTRIBUTION)
   47 FORMAT (21X,36HESTIMATED PRIOR PARAMETERS:   ALPHA=,G11.4,9H    BE
     1TA=,G11.4,11H  (OR TAU =,G11.4,1H))
   48 FORMAT (21X,37HPRIOR PROBABILITY INTERVAL:   LAMBDA(,F5.3,8H LEVEL
     1)=,G11.4,13H      LAMBDA(,F5.3,8H LEVEL)=,G11.4)
   49 FORMAT (104H1*** METHOD 2 *** -- MATCHING FIRST TWO DATA MOMENTS T
     1O THOSE OF THE MARGINAL POISSON-GAMMA DISTRIBUTION)
   50 FORMAT (69H1*** METHOD 3 *** -- MARGINAL MAXIMUM LIKELIHOOD ESTIMA
     1TION TECHNIQUE)
   51 FORMAT (8X,38HNEGATIVE PARAMETER ESTIMATES:  ALPHA= ,G11.4,8H  BET
     1A= ,G11.4/8X,86H   INVALID ESTIMATES --- ATTEMPT TO FIND ESTIMATES
     2 USING THE MAXIMUM LIKELIHOOD METHOD)
   52 FORMAT (20X,55HMAXIMUM LIKELIHOOD FOUND NO ESTIMATORS FOR BETA BET
     1WEEN,G15.7,6H   AND,G15.7)
   53 FORMAT (1H1)
      END
C
C
      SUBROUTINE VARS(ALPHA,BETA,ITYPE,VAR,IER)
C******************************* VARS **********************************
C*                                                                     *
C* PURPOSE:                                                            *
C* THIS SUBROUTINE ESTIMATES THE VARIANCES AND COVARIANCE OF THE       *
C* GAMMA PARAMETERS ALPHA AND BETA AS ESTIMATED BY THE METHOD OF MATCH-*
C* ING MOMENTS AND THE METHOD OF MAXIMUM LIKELIHOOD.  THE CORRELATION  *
C* BETWEEN ALPHA AND BETA IS ALSO COMPUTED.                            *
C*                                                                     *
C* SUBROUTINE VARIABLES:                                               *
C*    ALPHA  = THE ESTIMATED GAMMA PARAMETER ALPHA                     *
C*    BETA   = THE ESTIMATED GAMMA PARAMETER BETA                      *
C*    ITYPE  = CONTROL INDEX FOR METHOD BY WHICH ALPHA AND BETA WERE   *
C*             ESTIMATED: =0 IF THE METHOD OF PRIOR MATCHING MOMENTS   *
C*             WERE USED; =1 IF THE METHOD OF MARGINAL MATCHING MOM-   *
C*             ENTS WERE USED; =2 IF MAXIMUM LIKELIHOOD ESTIMATORS     *
C*             ARE SUPPLIED.                                           *
C*    VAR    = VECTOR WHICH RETURNS VARIANCES AND CORRELATION ESTI-    *
C*             MATES: VAR(1) = VAR(ALPHA), VAR(2)=VAR(BETA), VAR(3)=   *
C*             COVAR(ALPHA,BETA), VAR(4)=R                             *
C*    IER    = ERROR RETURN INDEX: =0 IF CORRECT ESTIMATES GIVEN; =1   *
C*             IF DETERMINANT OF H MATRIX OR INFORMATION MATRIX IS 0;  *
C*             =2 IF VAR(ALPHA) OR VAR(BETA) IS NOT POSITIVE           *
C*                                                                     *
C* COMMON BLOCK /DTA/ ID(200),NF(100),NT(100),NN(100),N,NDATA          *
C*    ID(I)  = IDENTIFICATION FOR I-TH COMPNENT (NOT USED)             *
C*    NF(I)  = NUMBER OF FAILURES FOR I-TH COMPONENT CLASS             *
C*    NT(I)  = TEST TIME FOR I-TH COMPONENT CLASS                      *
C*    NN(I)  = NUMBER OF COMPONENTS IN I-TH CLASS                      *
C*    N      = NUMBER OF COMPONETS (CALCULATED)                        *
C*    NDATA  = NUMBER OF COMPONENT CLASSES                             *
C*                                                                     *
C* SUBROUTINES NEEDED:  NONE                                           *
C*                                                                     *
C* WRITTEN BY J. K. SHULTIS, KANSAS STATE UNIVERSITY, 6/80             *
C*                                                                     *
C***********************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 VAR(4)
      COMMON/DTA/IDUM(200),NF(100),NT(100),NN(100),N,NDATA,IPRT,NITER
C
      IF (ITYPE.EQ.2) GO TO 3
C
C*** COMPUTE THE VARIANCES FOR THE MATCHING MOMENTS ESTIMATERS
C    COMPUTE ELEMENTS OF COVARIANCE MATRIX, SIGMA
      AB = ALPHA*BETA
      AB2 = BETA*AB
      A = ALPHA
      B = BETA
      SUMTI = 0.0
      SUMTI2 = 0.0
      SUMTI3 = 0.0
      IF (ITYPE.EQ.0) GO TO 30
      DO 1 I=1,NDATA
      TT = NT(I)
      SUMTI2 = SUMTI2 + NN(I)/(TT*TT)
      SUMTI3 = SUMTI3 + NN(I)/(TT*TT*TT)
    1 SUMTI = SUMTI+NN(I)/TT
   30 TN = SUMTI/N
      T2N = SUMTI2/N
      T3N = SUMTI3/N
      S1 = 1./(N-1.)
      CC = AB*(B+TN)
      DD = AB*(2.*B*B + 3.*B*TN + T2N)
      TT=6. + 3.*A
      FF = AB2*(3.*B*B*(A+2.)-AB2*(N-3.)*S1) + TN*2.*AB2*B*(TT-(N-3.)*S1
     1      *A) + T2N*AB2*(7. +3.*A - (N*N-2.*N+3.)*A*S1*S1) + T3N*AB
     2      + 2.*N*(TN*AB*S1)**2
C    CALCULATE ELEMENTS OF THE H MATRIX
      A = BETA
      B = ALPHA
      C = BETA*(BETA+TN*ITYPE)
      D = 2.*AB+ALPHA*TN*ITYPE
      DET2 = (A*D-C*B)**2
      IF (DET2.GT.0.0) GO TO 2
      IER = 1
      RETURN
C    CALCULATE VARIANCES AND COVARIANCE
    2 FACT = 1./(N*DET2)
      VAR(1) = FACT*(CC*D*D+FF*B*B-2.*DD*B*D)
      VAR(2) = FACT*(CC*C*C+FF*A*A-2.*DD*A*C)
      VAR(3) = FACT*(DD*(A*D+C*B)-FF*A*B-CC*C*D)
      GO TO 8
C
C*** COMPUTE THE VARIANCES FOR THE MAXIMUM LIKELIHOOD ESTIMATERS
C    CALCULATE THE SUMMATION TERMS IN THE DERIVATIVES
    3 F = 0.0
      SUMA2 = 0.0
      SUMB2 = 0.0
      SUMAB = 0.0
      DO 6 I=1,NDATA
      SUM = 0.0
      T = NT(I)
      IF = NF(I)
      IF (IF.EQ.0) GO TO 5
      DO 4 J=1,IF
    4 SUM = SUM+1./(ALPHA+J-1.)**2
    5 SUMA2 = SUMA2+NN(I)*SUM
      SUMB2 = SUMB2+NN(I)*(IF+ALPHA)*T**2/(1.+BETA*T)**2
      SUMAB = SUMAB+NN(I)*T/(1.+BETA*T)
    6 F = F+NN(I)*IF
C    CALCULATE SECOND DERIVATIVES
      A = SUMA2
      D = F/BETA**2-SUMB2
      C = SUMAB
      DET = A*D-C*C
C    CALCULATE THE VARIANCES AND COVARIANCE BOUNDS
      IF (DET.NE.0.0) GO TO 7
      IER = 1
      RETURN
    7 VAR(1) = D/DET
      VAR(2) = A/DET
      VAR(3) = -C/DET
C
C*** COMPUTE THE CORRELATION COEFFICIENT R
    8 IF ((VAR(1).GT.0.0).AND.(VAR(2).GT.0.0)) GO TO 9
      IER = 2
      VAR(4) = 0.0
      RETURN
    9 VAR(4) = VAR(3)/(DSQRT(VAR(1))*DSQRT(VAR(2)))
      IER = 0
      RETURN
      END
C
      SUBROUTINE POSTGM (A,B,PCT,EPS)
C***********************************************************************
C*                                                                     *
C* THIS SUBROUTINE PERFORMS AN ANALYSIS OF THE POSTERIOR GAMMA DIS-    *
C* TRIBUTION FOR EACH COMPONENT.  ARGUMENTS ARE: A= ALPHA PARAMETER    *
C* OF THE GAMMA PRIOR, B = BETA PARAMETER OF THE GAMMA PRIOR, AND PCT= *
C* THE PERCENTILE OF THE UPPER PROBABILITY INTERVAL FOR LAMBDA.        *
C*                                                                     *
C***********************************************************************
C
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 ID(100),LAM(100),LAML(100),LAMU(100),MEAN(100),MEDIAN
      COMMON /DTA/ID,NF(100),NT(100),NN(100),N,NDATA,IPRT,NITER
      COMMON /CLASS/LAM,LAML,LAMU
      EXTERNAL FGAM
C
C     PRINT TABLE HEADINGS
      PCCT = 1.0-2.0*PCT
      WRITE (IPRT,8)
      WRITE (IPRT,9)
      WRITE (IPRT,10) PCCT,PCCT
      WRITE (IPRT,11)
C
C     BEGIN POSTERIOR ANALYSIS OF EACH COMPONENT
      DO 1 I=1,NDATA
      T = NT(I)
      F = NF(I)
      TAU = 1./B
C     CALCULATE MOMENTS ABOUT ZERO
      FF = F+A
      TT = T+TAU
      ZM1 = FF/TT
      ZM2 = ZM1*(FF+1.)/TT
      ZM3 = ZM2*(FF+2.)/TT
      VAR = ZM2-ZM1*ZM1
      CM3 = ZM3-3.*ZM2*ZM1+2.*ZM1**3
      SKW = CM3/VAR**1.5
      MEAN(I) = ZM1
C
C     CALCULATE POSTERIOR PERCENTILES AND MEDIAN
      CALL GMPCTL (RTUP,RTLOW,PCT,FF,TT,EPS,0)
      CALL GMPCTL (MEDIAN,XX,0.5D0,FF,TT,EPS,0)
C
C     PRINT OUT VALUES FOR EACH COMPONENT
    1 WRITE (IPRT,12) ID(I),NF(I),NT(I),LAM(I),LAML(I),LAMU(I),ZM1,MEDIA
     1N,VAR,SKW,RTLOW,RTUP
      WRITE (IPRT,13)
C
C  CALCULATE MINIMUM AND MAXIMUM FAILURE RATES FOR HOMO. NODEL (HRTMAX &
C  HRTMIN) AND FOR COMPOUND MODEL (CRTMAX & CRTMIN)
      HRTMAX = LAM(1)
      HRTMIN = LAM(1)
      CRTMAX = MEAN(1)
      CRTMIN = CRTMAX
      IHMAX = 1
      IHMIN = 1
      ICMAX = 1
      ICMIN = 1
C      SEARCH FOR MIN AND MAX AMOUNG REST OF THE COMPONENTS
      DO 5 I=2,NDATA
      IF (HRTMAX.GE.LAM(I)) GO TO 2
      HRTMAX = LAM(I)
      IHMAX = I
    2 IF (HRTMIN.LE.LAM(I)) GO TO 3
      HRTMIN = LAM(I)
      IHMIN = I
    3 IF (CRTMAX.GE.MEAN(I)) GO TO 4
      CRTMAX = MEAN(I)
      ICMAX = I
    4 IF (CRTMIN.LE.MEAN(I)) GO TO 5
      CRTMIN = MEAN(I)
      ICMIN = I
    5 CONTINUE
C     PRINT OUT MIN AND MAX VALUES OF FAILURE RATES
      WRITE (IPRT,6)
      WRITE (IPRT,7) HRTMIN,IHMIN,ID(IHMIN),HRTMAX,IHMAX,ID(IHMAX),CRTMI
     1N,ICMIN,ID(ICMIN),CRTMAX,ICMAX,ID(ICMAX)
      RETURN
C
    6 FORMAT (/35H MINIMUM AND MAXIMUM FAILURE RATES:,17X,25HMIN. LAMBDA
     1 -- COMP.   ID,19X,25HMAX. LAMBDA -- COMP.   ID)
    7 FORMAT (4X,18HHOMOGENEOUS MODEL:,15X,2(15X,G11.4,I6,4X,A8)/4X,32HC
     1OMPOUND MODEL (POSTERIOR MEAN):,2X,2(14X,G11.4,I6,4X,A8,1X))
    8 FORMAT (/////97H ANALYSIS OF EACH COMPONENT WITH A HOMOGENEOUS MOD
     1EL  AND WITH A COMPOUND MODEL (GAMMA POSTERIOR)/)
    9 FORMAT (24H ---COMPONENT DATA----  ,8(1H-),17HHOMOGENEOUS MODEL,9(
     11H-),2X,22(1H-),26HCOMPOUND MODEL (POSTERIOR),23(1H-))
   10 FORMAT (4X,2HID,4X,10HNO.   TIME,6X,6HLAMBDA,4X,F5.3,16H CONFIDENC
     1E INT.,6X,4HMEAN,7X,6HMEDIAN,5X,8HVARIANCE,4X,8HSKEWNESS,3X,F5.3,1
     27H PROBABILITY INT.)
   11 FORMAT (9X,4HFAIL,14X,5H(F/T),6X,5HLOWER,7X,5HUPPER,7X,6HLAMBDA,6X
     1,6HLAMBDA,30X,5HLOWER,7X,5HUPPER/1X,130(1H-))
   12 FORMAT (1H ,A8,I4,I8,2X,9(1X,G11.4))
   13 FORMAT (1H ,130(1H-))
      END
C
      SUBROUTINE GMPCTL (LAMUP,LAMLOW,PCT,ALPHA,TAU,ACC,ICLASS)
C***********************************************************************
C*                                                                     *
C* THIS SUBROUTINE CALCULATES THE UPPER AND LOWER CONFIDENCE OR PROBA- *
C* BILITY INTERVALS FOR THE HOMOGENEOUS CLASSICAL FAILURE RATE PROBLEM *
C* OF FOR THE BAYESIAN POSTERIOR GAMMA DISTRIBUTION.                   *
C*                                                                     *
C* INPUT VARIABLES:                                                    *
C*    LAMUP  = UPPER INTERVAL BOUND                                    *
C*    LAMLOW = LOWER INTERVAL BOUND                                    *
C*    PCT    = FRACTION OF AREA IN EACH TAIL OF THE DISTRIBUTION       *
C*             (< OR = HALF OF THE TOTAL AREA 0.5)                     *
C*    ALPHA  = ALPHA PARAMETER OF THE APPROPRIATE INCOMPLETE GAMMA FN  *
C*    TAU    = TAU PARAMETER FOR THE APPROPRIATE GAMMA DISTRIBUTION    *
C*    ACC    = ACCURACY PARAMETER FOR THE INTERVAL BOUNDS (ABSOLUTE    *
C*             ERROR)                                                  *
C*    ICLASS = 0 FOR CONFIDENCE INTERVAL FROM THE POISSON; =1 FOR THE  *
C*             INTERVAL FROM A GAMMA DISTRIBUTION                      *
C*                                                                     *
C* WRITTEN BY J. K. SHULTIS, KANSAS STATE UNIVERSITY, 3/80             *
C*                                                                     *
C***********************************************************************
      REAL*8 LAMUP,LAMLOW,PCT,ALPHA,TAU,ACC,ALPH,FNVAL,PCTL,EPS,TTT,UPP
      COMMON /DTA/IDUMMY(502),IPRT,NITER
      COMMON /FGM/ALPH,TTT,PCTL,EPS
      EXTERNAL FGAM
      ALPH = ALPHA
      TTT = TAU
      EPS = ACC
      IF (ALPHA.EQ.0.0) GO TO 1
C
C     CALCULATE UPPER BOUND FOR LAMBDA PERCENTILES BY THE TSCHEBYSCHEFF
C     INEQUALITY
      PCTL = DSQRT(ALPHA)
      UPP = 1.0/DSQRT(PCT)
      IF (PCTL.GT.UPP) UPP = PCTL
      UPP = (ALPHA+PCTL*UPP)/TAU
C
C     CALCULATE LOWER PERCENTILE
      PCTL = PCT
      CALL DRTMI (LAMLOW,FNVAL,FGAM,0.0D0,UPP,ACC,NITER,IER)
      IF (IER.NE.0) WRITE (IPRT,2) IER
C
C     CALCULATE UPPER PERCENTILE
      IF (ICLASS.EQ.1) ALPH = ALPHA+1.0
      PCTL = 1.0-PCT
      CALL DRTMI (LAMUP,FNVAL,FGAM,0.0D0,UPP,ACC,NITER,IER)
      IF (IER.NE.0) WRITE (IPRT,3) IER
      RETURN
C
C     SPECIAL CASE IF ALPHA = 0
    1 LAMLOW = 0.0
      LAMUP = -DLOG(PCT)/TAU
      RETURN
    2 FORMAT (61H ERROR IN CALCULATION OF LOWER BOUND IN GMPCTL -- RETUR
     1N CODE,I3)
    3 FORMAT (61H ERROR IN CALCULATION OF UPPER BOUND IN GMPCTL -- RETUR
     1N CODE,I3)
      END
      REAL FUNCTION FGAM*8(X)
C***********************************************************************
C*                                                                     *
C* THIS FUNCTION IS NEEDED BY SUBROUTINE POSTGM TO EVALUATE THE UPPER  *
C* PROBABILITY INTERVAL FOR LAMBDA. THE INCOMPLETE GAMMA FUNCTION IS   *
C* EVALUATED FOR A SPECIFIED UPPER LIMIT.                              *
C*                                                                     *
C***********************************************************************
      REAL*8 ALPHA,PCT,X,Y(2),EPS,XX,TAU
      COMMON /DTA/IDUMMY(502),IPRT,NITER
      COMMON /FGM/ALPHA,TAU,PCT,EPS
      XX = TAU*X
      CALL GAMIC (XX,ALPHA,EPS,1,Y,NZ)
      IF (NZ.NE.0) WRITE (IPRT,1)
      FGAM = PCT-Y(1)
      RETURN
    1 FORMAT (51H ERROR IN EVALUATION OF INCOMPLETE GAMMA FN IN FGAM)
      END
      SUBROUTINE MLGAMP (FN,ALPHA,BETA)
C***********************************************************************
C*                                                                     *
C* THIS SUBROUTINE EVALUATES ALPHA AND THE FUNCTION FN WHOSE ZERO      *
C* GIVES THE VALUE OF BETA FOR THE MAXIMUM LIKELIHOOD ESTIMATORS.      *
C*                                                                     *
C***********************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON /DTA/IDUMMY(200),NF(100),NT(100),NN(100),N,NDATA,IPRT,NITER
C
C     CALCULATE ALPHA AS A FUNCTION OF BETA
      SUM1 = 0.
      SUM2 = 0.
      FF = 0.
      DO 1 J=1,NDATA
      FF = FF+NF(J)*NN(J)
      A = NN(J)*NT(J)/(1.+BETA*NT(J))
      SUM1 = SUM1+A
    1 SUM2 = SUM2+A*NF(J)
      ALPHA = (FF/BETA-SUM2)/SUM1
C
C     CALCULATE F(ALPHA,BETA)
      FN = 0.
      DO 4 J=1,NDATA
      A = DLOG(1.+BETA*NT(J))
      SUM = 0.
      NNN = NF(J)
      IF (NNN.EQ.0) GO TO 3
      DO 2 K=1,NNN
    2 SUM = SUM+1./(ALPHA+K-1.)
    3 FN = FN+(A-SUM)*NN(J)
    4 CONTINUE
      RETURN
      END
      REAL FUNCTION FN*8(X)
C***********************************************************************
C*                                                                     *
C* THIS FUNCTION EVALUATES THE AUXILARY LIKELIHOOD FUNCTION FN WHOSE   *
C* ZERO GIVES THE VALUE OF THE BETA PARAMETER.  THIS SUBROUTINE SIMPLY *
C* CALLS MLGAMP WHERE THE ACTUAL EVALUATION IS PERFORMED.              *
C*                                                                     *
C***********************************************************************
      REAL*8 ALPHA,X
      CALL MLGAMP (FN,ALPHA,X)
      RETURN
      END
      SUBROUTINE DRTMI (X,F,FCT,XLI,XRI,EPS,IEND,IER)
C******************************* DRTMI *********************************
C*                                                                     *
C* THIS SUBROUTINE SOLVES A NONLINEAR EQUATION OF THE FORM FCT(X)=0    *
C* BY THE MUELLER'S ITERATION METHOD.                                  *
C* THIS PROGRAM IS TAKEN FROM THE IBM SSP (RTMI) AND MODIFIED TO WORK  *
C* IN DOUBLE PRECISION.                                                *
C*                                                                     *
C*    INPUT VARIABLES:                                                 *
C*        X    = RETURNED VALUE OF THE ZERO                            *
C*        F    = VALUE OF THE FUNCTION FCT AT THE ZERO                 *
C*        FCT  = EXTERNAL FUNCTION WHOSE ZERO IS TO BE FOUND           *
C*        XLI  = LEFT BOUNDARY OF THE X-AXIS TO BE SEARCHED FOR ROOT   *
C*        XRI  = RIGHT BOUNDARY OF X-AXIS TO BE SEARCHED FOR ROOT      *
C*        EPS  = ACCURACY OF DESIRED RESULT                            *
C*        IEND = MAXIMUM NUMBER OF ITERATIONS TO BE USED               *
C*        IER  = ERROR RETURN CODE; =0 IF DESIRED ACCURACY ACHIEVED    *
C*                                                                     *
C***********************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
C
C
C     PREPARE ITERATION
      IER = 0
      XL = XLI
      XR = XRI
      X = XL
      TOL = X
      F = FCT(TOL)
      IF (F) 1,16,1
    1 FL = F
      X = XR
      TOL = X
      F = FCT(TOL)
      IF (F) 2,16,2
    2 FR = F
      IF (DSIGN(1.D0,FL)+DSIGN(1.D0,FR)) 25,3,25
C
C     BASIC ASSUMPTION FL*FR LESS THAN 0 IS SATISFIED.
C     GENERATE TOLERANCE FOR FUNCTION VALUES.
    3 I = 0
      TOLF = 100.*EPS
C
C
C     START ITERATION LOOP
    4 I = I+1
C
C     START BISECTION LOOP
      DO 13 K=1,IEND
      X = .5*(XL+XR)
      TOL = X
      F = FCT(TOL)
      IF (F) 5,16,5
    5 IF (DSIGN(1.D0,F)+DSIGN(1.D0,FR)) 7,6,7
C
C     INTERCHANGE XL AND XR IN ORDER TO GET THE SAME SIGN IN F AND FR
    6 TOL = XL
      XL = XR
      XR = TOL
      TOL = FL
      FL = FR
      FR = TOL
    7 TOL = F-FL
      A = F*TOL
      A = A+A
      IF (A-FR*(FR-FL)) 8,9,9
    8 IF (I-IEND) 17,17,9
    9 XR = X
      FR = F
C
C     TEST ON SATISFACTORY ACCURACY IN BISECTION LOOP
      TOL = EPS
      A = DABS(XR)
      IF (A-1.) 11,11,10
   10 TOL = TOL*A
   11 IF (DABS(XR-XL)-TOL) 12,12,13
   12 IF (DABS(FR-FL)-TOLF) 14,14,13
   13 CONTINUE
C     END OF BISECTION LOOP
C
C     NO CONVERGENCE AFTER IEND ITERATION STEPS FOLLOWED BY IEND
C     SUCCESSIVE STEPS OF BISECTION OR STEADILY INCREASING FUNCTION
C     VALUES AT RIGHT BOUNDS. ERROR RETURN.
      IER = 1
   14 IF (DABS(FR)-DABS(FL)) 16,16,15
   15 X = XL
      F = FL
   16 RETURN
C
C     COMPUTATION OF ITERATED X-VALUE BY INVERSE PARABOLIC INTERPOLATION
   17 A = FR-F
      DX = (X-XL)*FL*(1.+F*(A-TOL)/(A*(FR-FL)))/TOL
      XM = X
      FM = F
      X = XL-DX
      TOL = X
      F = FCT(TOL)
      IF (F) 18,16,18
C
C     TEST ON SATISFACTORY ACCURACY IN ITERATION LOOP
   18 TOL = EPS
      A = DABS(X)
      IF (A-1.) 20,20,19
   19 TOL = TOL*A
   20 IF (DABS(DX)-TOL) 21,21,22
   21 IF (DABS(F)-TOLF) 16,16,22
C
C     PREPARATION OF NEXT BISECTION LOOP
   22 IF (DSIGN(1.D0,F)+DSIGN(1.D0,FL)) 24,23,24
   23 XR = X
      FR = F
      GO TO 4
   24 XL = X
      FL = F
      XR = XM
      FR = FM
      GO TO 4
C     END OF ITERATION LOOP
C
C
C     ERROR RETURN IN CASE OF WRONG INPUT DATA
   25 IER = 2
      RETURN
      END
C
C
C
      SUBROUTINE LKLIHD(PROB,LNPROB,PARM1,PARM2,ITYPE)
C**************************** LKLIHD ***********************************
C*                                                                     *
C*  PURPOSE:                                                           *
C*    THIS SUBROUTINE GENERATES THE VALUE OF THE LIKELIHOOD AND ITS    *
C*    LOGARITHM FOR EACH COMPONENT FOR EITHER A HOMOGENEOUS POISSON    *
C*    MODEL OR FOR A COMPOUND POISSON-GAMMA MODEL.                     *
C*                                                                     *
C*  INPUT PARAMETERS:                                                  *
C*    PROB  = VALUE OF THE LIKELIHOOD RETURNED                         *
C*    LNPROB= NATURAL LOG OF LIKELIHOOD (RETURNED VALUE)               *
C*    ITYPE = TYPE OF MODEL TO BE USED: =0 USE THE HOMOGENEOUS         *
C*            POISSON MODEL; =1 USE THE COMPOUND POISSON-GAMMA MODEL   *
C*    PARM1 = LAMBDA (FAILURE RATE) FOR THE HOMOGENEOUS MODEL;         *
C*            ALPHA (GAMMA PAERAMETER) FOR THE COMPOUND MODEL          *
C*    PARM2 = TAU (SECOND GAMMA PARAMETER FOR THE COMPOUND MODEL;      *
C*            DUMMY VARIABLE (NOT USED) FOR THE HOMOGENEOUS MODEL      *
C*                                                                     *
C*  SUBROUTINES NEEDED:   DXP                                          *
C*                                                                     *
C*  WRITTEN BY J. K. SHULTIS, KANSAS STATE UNIVERSITY, 6/80            *
C*                                                                     *
C***********************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 PROB,ID(100),LNPOIS,PARM1,PARM2,LNMARG,LNPROB
      COMMON/DTA/ID,NF(100),NT(100),NN(100),N,NDATA,IPRT,NITER
      LNMARG(F,T) = DLGAMA(F+PARM1)-DLGAMA(F+1.)-DLGAMA(PARM1)+F*DLOG(T)
     1-(F+PARM1)*DLOG(T+PARM2)+PARM1*DLOG(PARM2)
      LNPOIS(F,T) = F*DLOG(PARM1*T)-PARM1*T-DLGAMA(F+1.)
C
      LNPROB = 0.0
      IF (ITYPE.GT.0) GO TO 2
C*** CALCULATE LIKELIHOOD FOR THE HOMOGENEOUS MODEL
      DO 1 I=1,NDATA
      F = NF(I)
      T = NT(I)
    1 LNPROB = LNPROB+LNPOIS(F,T)*NN(I)
      PROB = DXP(LNPROB)
      RETURN
C
C*** CALCULATE LIKELIHOOD  FOR COMPOUND MODEL
    2 DO 3 I=1,NDATA
      F = NF(I)
      T = NT(I)
    3 LNPROB = LNPROB+LNMARG(F,T)*NN(I)
      PROB = DXP(LNPROB)
      RETURN
      END
C
C
      SUBROUTINE CHISQ(H,EXPMIN,MAXGPS,LDF,INTER,EPS,CHI2,NDF,
     1 SIG,NZ)
C
C********************************* CHISQ *******************************
C*                                                                     *
C* THIS PROGRAM PERFORMS A CHI-SQUARED GOODNESS OF FIT TEST ON GIVEN   *
C* FAILURE RATE ATTRIBUTE DATA WITH A COMPOUND POISSON-GAMMA FAILURE   *
C* RATE MODEL.  DIFFERENT TEST TIMES FOR THE COMPONENTS ARE ALLOWED.   *
C* FAILURE DATA ARE SUPPLIED TO THE SUBPROGRAM THROUGH THE COMMON      *
C* BLOCK 'DTA'.                                                        *
C*                                                                     *
C* INPUT PARAMETERS:                                                   *
C*    H     = EXTERNAL FUNCTION WHICH GIVES THE PROBABILITY OF         *
C*            OBTAINING FF FAILURES IN TEST TIME TT                    *
C*    EXPMIN= MINIMUM VALUE OF THE EXPECTED NUMBER OF FAILURES IN      *
C*            EACH CLASS                                               *
C*    MAXGPS= MAXIMUM NUMBER OF CLASSES DESIRED FOR THE CHI-SQUARED    *
C*            TEST                                                     *
C*    LDF   = NUMBER OF DEGREES OF FREEDOM TO BE SUBTRACTED FROM       *
C*            THE TOTAL NUMBER OF CLASSES TO GIVE THE NUMBER OF        *
C*            DEGREES OF FREEDOM FOR THE FIT TEST                      *
C*    INTER = 0 IF NO INTERMEDIATE OUTPUT IS DESIRED; =1 IF INTER-     *
C*            MEDIATE OUTPUT IS DESIRED                                *
C*    EPS   = RELATIVE ACCURACY OF THE SIGNIFICANCE LEVEL, I.E.        *
C*            1.D-S SPECIFIES S SIGNIFICANT FIGURES ARE DESIRED        *
C*                                                                     *
C* OUTPUT PARAMETERS:                                                  *
C*    CHI2  = VALUE OF THE CHI-SQUARED TEST STATISTIC                  *
C*    NDF   = NUMBER OF DEGREES OF FREEDOM USED IN THE TEST            *
C*    SIG   = SIGNIFICANCE LEVEL FOR THE CHI-SQUARED STATISTIC         *
C*    NZ    = ERROR RETURN CODE: =0 NORMAL RETURN, =-1 EXPECTATION     *
C*            VECTOR MUST BE MADE LARGER, >0 ERROR IN CALCULATION      *
C*            OF THE SIGNIFICANCE LEVEL (SEE SUBROUTINE 'GAMIC'        *
C*            ERROR RETURN CODES)                                      *
C*                                                                     *
C* COMMON BLOCK   /DTA/ ID,NF,NT,NN,N,NDATA,IPRT,NITER                 *
C*    ID (800 BYTES), CONTAINS IDENTIFICATION LABELS FOR COMPONENTS    *
C*                    (NOT USED IN PROGRAM)                            *
C*    NF (400 BYTES), CONTAINS NUMBER OF FAILURES FOR THE COMPONENTS   *
C*    NT (400 BYTES), CONTAINS TEST TIME FOR COMPONENTS                *
C*    NN (400 BYTES), NUMBER OF COMPONENTS WITH SAME NF(I) AND NT(I)   *
C*    N  (4 BYTES),   TOTAL NUMBER OF COMPONENTS                       *
C*    NDATA (4 BYTES),NUMBER OF COMPONENT DATA ENTRIES, I.E. NUMBER OF *
C*                    VALUES FOR NF(I), NT(I), AND NN(I)               *
C*    IPRT (4 BYTES), UNIT NUMBER FOR INTERMEDIATE OUTPUT              *
C*    NITER (4 BYTES), NOT USED IN SUBROUTINE                          *
C*                                                                     *
C* SUBROUTINES NEEDED:  H, GAMIC                                       *
C*                                                                     *
C* WRITTEN BY J. K. SHULTIS, KANSAS STATE UNIVERSITY,  6/80            *
C*                                                                     *
C***********************************************************************
C
      REAL*8 ID(100),SUM,H,E(100),EHAT(100),CHI2,EXPMIN,
     1  Y(2),EPS,SIG,FF,X,TT,ALP,DIFF,CHI(50)
      DIMENSION NFHAT(100),NOB(100)
      COMMON/DTA/ID,NF(100),NT(100),NN(100),N,NDATA,IPRT,NITER
C
C*** CALCULATE TOTAL NUMBER OF OBSERVED FAILURES
      KMAX = 100
      DO 1 I=1,KMAX
    1 NOB(I) = 0.0
      NFTOT = 0
      DO 2 I=1,NDATA
      NFF = NF(I)+1
      NFTOT = NFTOT+NN(I)
    2 NOB(NFF) = NOB(NFF)+NN(I)
C
C*** CALCULATE THE EXPECTATIONS FOR EACH T(I)
      SUM = 0.0
      IF (INTER.EQ.1) WRITE (IPRT,28)
C
      DO 4 K=1,KMAX
      E(K) = 0.0
      IF = K-1
      FF = IF
      DO 3 I=1,NDATA
      TT = NT(I)
    3 E(K) = E(K)+H(FF,TT)*NN(I)
      SUM = SUM+E(K)
      NFTOT = NFTOT-NOB(K)
      IF (INTER.EQ.1) WRITE (IPRT,29) IF,NOB(K),E(K),SUM
      SUMM = N-SUM
      IF (SUMM.GT.EXPMIN) GO TO 4
      LAST = K
      KSTAR = K-2
      E(K) = N-(SUM-E(K))
      IF = IF+1
      IF (INTER.EQ.1) WRITE (IPRT,30) IF,NFTOT,SUMM,N
      GO TO 5
    4 CONTINUE
C  RETURN IF THE SIZE OF THE EXPECTATION VECTOR, E(K), > 100
      NZ = -1
      RETURN
C
C*** GROUP THE EXPECTATIONS SO THAT AT LEAST EXPMIN ARE IN EACH BIN
    5 KSTRT = 1
      LAST = KSTAR+2
      KG = 1
      NFHAT(1) = -1
    6 SUM = 0.0
      DO 7 J=KSTRT,LAST
      SUM = SUM+E(J)
      IF (SUM.LT.EXPMIN) GO TO 7
      EHAT(KG) = SUM
      NFHAT(KG+1) = J-1
      GO TO 8
    7 CONTINUE
    8 KSTRT = KSTRT+(NFHAT(KG+1)-NFHAT(KG))
      IF (NFHAT(KG+1).GE.(KSTAR+1)) GO TO 9
      KG = KG+1
      GO TO 6
C PRINT OUT VALUES FOR EACH CLASS
    9 IF ((KG-LDF).LE.1) GO TO 27
      IF (INTER.EQ.0) GO TO 11
      WRITE (IPRT,31) EXPMIN
      KK = KG-1
      DO 10 I=1,KK
      II = NFHAT(I)+1
   10 WRITE (IPRT,32) I,II,NFHAT(I+1),EHAT(I)
      II = NFHAT(KG)+1
      WRITE (IPRT,33) I,II,EHAT(KG)
C
C  COLLAPSE CLASSES FURTHER INTO MAXGPS BINS
   11 NEXTRA = KG-MAXGPS
      IF (NEXTRA) 19,19,12
   12 DO 18 I=1,NEXTRA
C FIND THE SMALLEST EXPECTATION AND BIN NUMBER
      EMIN = EHAT(1)
      KMIN = 1
      DO 13 K=2,KG
      IF (EHAT(K).GT.EMIN) GO TO 13
      EMIN = EHAT(K)
      KMIN = K
   13 CONTINUE
C FIND THE SMALLEST NEIGHBOUR (I.E. NEIGHBOURING BIN WITH SMALLEST EXP.
      IF (KMIN.GT.1) GO TO 14
      KNEXT = 2
      GO TO 16
   14 IF (KMIN.LT.KG) GO TO 15
      KNEXT = KG-1
      GO TO 16
   15 KNEXT = KMIN+1
      IF (EHAT(KMIN-1).LT.EHAT(KNEXT)) KNEXT = KMIN-1
C COMBINE THE SMALLEST CLASS WITH ITS SMALLEST NEIGHBOUR
   16 EHAT(KG+1) = 0.0
      KSTRT = KMIN+(KNEXT-KMIN+1)/2
      EHAT(KSTRT-1) = EHAT(KMIN)+EHAT(KNEXT)
      DO 17 K=KSTRT,KG
      EHAT(K) = EHAT(K+1)
   17 NFHAT(K) = NFHAT(K+1)
   18 KG = KG-1
C
C*** CALCULATE THE OBSERVED NUMBER IN EACH BIN
C INITIALIZE THE OBSERVED VECTOR
   19 DO 20 I=1,LAST
   20 NOB(I) = 0
      DO 22 I=1,NDATA
      NFF = NF(I)
      KK = KG-1
      DO 21 K=1,KK
      IF (NFF.GT.NFHAT(K+1)) GO TO 21
      NOB(K) = NOB(K)+NN(I)
      GO TO 22
   21 CONTINUE
      NOB(KG) = NOB(KG)+NN(I)
   22 CONTINUE
C
C CALCULATE CHI-SQUARED CONTRIBUTION FOR EACH BIN
      CHI2 = 0.0
      DO 23 I=1,KG
      CHI(I) = (NOB(I)-EHAT(I))**2/EHAT(I)
   23 CHI2 = CHI2+CHI(I)
C PRINT OUT THE COLLAPSED GROUP RESULTS AND OBSERVED VALUES
      IF (INTER.EQ.0) GO TO 25
      KK = KG-1
      WRITE (IPRT,34) MAXGPS
      DO 24 I=1,KK
      II = NFHAT(I)+1
      DIFF = EHAT(I)-NOB(I)
   24 WRITE (IPRT,35) I,II,NFHAT(I+1),EHAT(I),NOB(I),DIFF,CHI(I)
      II = NFHAT(KG)+1
      DIFF = EHAT(KG)-NOB(KG)
      WRITE (IPRT,36) KG,II,EHAT(KG),NOB(KG),DIFF,CHI(KG)
C
C*** CALCULATE THE SIGNIFICANCE LEVEL
   25 NDF = KG-LDF
      X = CHI2*0.5
      ALP = NDF*0.5
      CALL GAMIC (X,ALP,EPS,1,Y,NZ)
      SIG = 1.0-Y(1)
      WRITE (IPRT,37) CHI2
      WRITE (IPRT,38) NDF,SIG
   26 RETURN
C
   27 WRITE (IPRT,39)
      NZ = 2
      RETURN
   28 FORMAT (////48H INTERMEDIATE OUTPUT FOR CHI-SQUARED TEST OF FIT/4X
     1,9HNO. FAIL.,4X,8HOBSERVED,4X,8HEXPECTED,4X,16HCUMULATIVE EXPT.)
   29 FORMAT (6X,I3,11X,I3,G17.5,G15.5)
   30 FORMAT (4X,2H>=,I3,11X,I3,G17.5,I8)
   31 FORMAT (///68H COLLAPSE CLASSES SO THAT EACH CLASS HAS AN EXPECTAT
     1ION GREATER THAN,G11.4/22H CLASS    NO. FAILURES,7X,11HEXPECTATION
     2)
   32 FORMAT (1H ,I4,6X,I3,3H - ,I3,9X,G12.5)
   33 FORMAT (1H ,I4,9X,3H>= ,I3,9X,G12.5)
   34 FORMAT (///67H COLLAPSE THE NUMBER OF CLASSES SO TOTAL IS LESS THA
     1N   OR EQUAL TO,I3/85H GROUP   NUMBER OF FAILURES   EXPECTATIONS
     2 OBSERVED   EXP-OBS   CHI-SQ CONTRIBUTION)
   35 FORMAT (I5,6X,I3,3H - ,I3,10X,G12.5,I9,5X,G12.5,G15.5)
   36 FORMAT (1H ,I4,9X,3H>= ,I3,10X,G12.5,I9,5X,G12.5,G15.5)
   37 FORMAT (//24H CHI-SQUARED STATISTIC =,G15.6)
   38 FORMAT (74H PROBABILITY OF OBTAINING A VALUE AT LEAST AS LARGE AS
     1THIS STATISTIC WITH,I4,20H DEGREES OF FREEDOM,/46H WHEN THE NULL H
     2YPOTHESIS IS TRUE, IS EQUAL TO,G10.3)
   39 FORMAT (50H TOO FEW CLASSES TO PEFORM CHI-SQUARED TEST OF FIT)
      END
C
C
C
      SUBROUTINE KSFIT(H,D,IOUT,SIG1,SIG2)
C****************************** KSFIT **********************************
C*                                                                     *
C* PURPOSE:                                                            *
C* THIS SUBROUTINE CALCULATES THE KOLMOGOROV-SMIRNOV GOODNESS-OF-FIT   *
C* STATISTIC BETWEEN GIVEN FAILURE-RATE ATTRIBUTE DATA AND A SPECIFIED *
C* FAILURE RATE STATISTICAL MODEL.  BOUNDS FOR THE SIGNIFICANCE LEVEL  *
C* OF THE FIT STATISTIC ARE ESTIMATED FROM THE EXACT BOUNDS FOR SMALL  *
C* SAMPLE SIZES ( <51 ).                                               *
C*                                                                     *
C* SUBROUTINE VARIABLES:                                               *
C*    H     = NAME OF EXTERNAL FUNCTION WHICH GIVES THE PROBABILITY    *
C*            OF OBSERVING FF FAILURES IN TIME TT, I.E. H(FF,TT)       *
C*    D     = VALUE OF THE GOODNESS-OF-FIT STATISTIC                   *
C*    IOUT  = PARAMETER CONTROLLING INTERMEDIATE OUTPUT; IF = 0 NO     *
C*            INTERMEDIATE OUTPUT GIVEN; IF > 0 INTERMEDIATE OUTPUT    *
C*            PRINTED                                                  *
C*    SIG1  = LOWER BOUND ON THE SIGNIFICANCE LEVEL                    *
C*    SIG2  = UPPER BOUND ON THE SIGNIFICANCE LEVEL                    *
C*                                                                     *
C* COMMON BLOCK /DTA/ ID(100),NF(100),NT(100),NN(100),N,NDATA,IPRT     *
C*    THIS BLOCK SUPPLIES THE FAILURE DATA AGAINST WHICH THE MODEL     *
C*    IS TO BE TESTED.                                                 *
C*      ID(I)   = IDENTIFICATION FOR I-TH COMPONENT (REAL*8)           *
C*      NF(I)   = NUMBER OF FAILURES FOR I-TH COMPONENT CLASS          *
C*      NT(I)   = TEST TIME FOR I-TH COMPONENT CLASS                   *
C*      NN(I)   = NUMBER OF COMPONENTS IN I-TH COMPONENT CLASS         *
C*      N       = TOTAL NUMBER OF COMPONENTS                           *
C*      NDATA   = NUMBER OF COMPONENT CLASSES                          *
C*      IPRT    = UNIT NUMBER FOR PRINTED OUTPUT (NOT USED)            *
C*                                                                     *
C* SUBROUTINES NEEDED:  DKSLVL, KSSUM, DXP                             *
C*                      REAL FUNCTION H(FF,TT) MUST BE SUPPLIED        *
C*                                                                     *
C* WRITTEN BY J. K. SHULTIS, KANSAS STATE UNIVERSITY, 6/80             *
C*                                                                     *
C***********************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 ID(100)
      DIMENSION G(150)
      COMMON/DTA/ID,NF(100),NT(100),NN(100),N,NDATA,IPRT,NITER
C
      NSIZE = 150
      SIG1 = 0.0
      SIG2 = 1.0
      SUMOB = 0.0
      SUME = 0.0
      FACT = 1./N
      D = 0.0
C
C CALCULATE THE CUMULANT PROBABILITIES FOR THE OBSERVED AND EXPECTED FAI
C FAILURES
      IF (IOUT.GT.0) WRITE (IPRT,8)
      KK = 0
    1 KK = KK+1
      IF (KK.GT.NSIZE) GO TO 3
      K = KK-1
      FF = K
      SUMK = 0.0
      DO 2 I=1,NDATA
C  CALCULATE CUMULANT OF OBSERVED
      IF (NF(I).EQ.K) SUMOB = SUMOB+NN(I)
C  CALCULATE CUMULANT OF EXPECTED
      TT = NT(I)
    2 SUMK = SUMK+H(FF,TT)*NN(I)
      SUME = SUME+SUMK
      G(KK) = SUME*FACT
      SKK = SUMOB*FACT
C  CALCULATE MAXIMUM DEVIATION
      DIFF = DABS(G(KK)-SKK)
      IF (D.LT.DIFF) D = DIFF
      IF (IOUT.GT.0) WRITE (IPRT,9) K,SKK,G(KK)
C  TEST IF G(K) IS SUFFICIENTLY CLOSE TO UNITY TO TRUNCATE SEQUENCE
      IF (((1.-G(KK)).LT.D).AND.((1.-SKK).LT.D)) GO TO 4
      GO TO 1
C  NOT ENOUGH STORAGE FOR G(K) CUMULANTS - VECTOR G(K) MUST BE BIGGER
    3 WRITE (IPRT,10) NSIZE,G(NSIZE)
      IF (ILVL.LT.0) WRITE (IPRT,11) D
      RETURN
C
C TEST IF SUFFICIENT TERMS HAVE BEEN CALCULATED
    4 D1 = 1.-D
      NEND = N*D1
      IF (NEND-N*D1) 6,5,5
    5 IF (G(KK).GE.(1.-FACT)) GO TO 7
      GO TO 1
    6 IF (G(KK).GE.(D+NEND*FACT)) GO TO 7
      GO TO 1
    7 IF (IOUT.GT.0) WRITE (IPRT,12) D
C
C CALCULATE SIGNIFICANCE LEVEL
      NTOT = N
      CALL DKSLVL (D,NTOT,G,NSIZE,IOUT,SIG1,SIG2,ILVL)
      IF (ILVL.EQ.0) WRITE (IPRT,13) D,SIG1,SIG2
      RETURN
    8 FORMAT (//9X,1HK,8X,33HCUMULANTS:  OBSERVED     EXPECTED)
    9 FORMAT (1H ,I9,12X,G20.6,G13.6)
   10 FORMAT (/58H IN K-S FIT TEST, VECTOR G(K) MUST BE INCREASE IN SIZE
     1: G(,I3,3H) =,G15.6)
   11 FORMAT (56H DISCRETE KOLMOGOROV-SMIRNOV GOODNESS-OF-FIT STATISTIC
     1=,G12.5)
   12 FORMAT (/36H KOLMOGOROV-SMIRNOV FIT STATISTIC D=,G15.6)
   13 FORMAT (//56H DISCRETE KOLMOGOROV-SMIRNOV GOODNESS-OF-FIT STATISTI
     1C =,G12.5/104H PROBABILITY OF OBTAINING A VALUE AT LEAST AS LARGE
     2AS THIS STATISTIC, WHEN THE NULL HYPOTHESIS IS TRUE,,/11H IS BETWE
     3EN,G11.3,4H AND,G11.3)
      END
C
C
C
      SUBROUTINE DKSLVL(D,N,G,MAXG,IOUT,PMIN,PMAX,ILVL)
C******************************* DKSLVL ********************************
C*                                                                     *
C* PURPOSE:                                                            *
C* THIS SUBROUTINE CALCULATES BOUNDS FOR THE SIGNIFICANCE LEVEL FOR    *
C* A SPECIFIED KOLMOGOROV-SMIRNOV GOODNESS-OF-FIT STATISTIC FOR A      *
C* DISCRETE PROBABILITY MODEL. UPPER AND LOWER BOUNDS ARE COMPUTED     *
C* ONLY IF THE NUMBER OF DATA IS LESS THAN 51.                         *
C*                                                                     *
C* SUBROUTINE VARIABLES:                                               *
C*    D     = SPECIFIED K-S DISCRETE GOODNESS-OF-FIT STATISTIC         *
C*    N     = TOTAL NUMBER OF COMPONENTS IN SAMPLE                     *
C*    G     = INPUT VECTOR GIVING EXPECTATIONS, I.E. G(K+1) EXPECT-    *
C*            ATION OF OBSERVING K OR FEWER FAILURES FOR SAMPLE        *
C*    MAXG  = SIZE OF G VECTOR                                         *
C*    IOUT  = PARAMETER CONTROLLING INTERMEDIATE OUTPUT: IF = 0 NO     *
C*            INTERMEDIATE OUTPUT PRODUCED; IF > 0 INTERMEDIATE OUT-   *
C*            PUT PRODUCED ON UNIT NUMBER 'IOUT'                       *
C*    PMIN  = LOWER BOUND ON THE SIGNIFICANCE LEVEL                    *
C*    PMAX  = UPPER BOUND ON THE SIGNIFICANCE LEVEL                    *
C*    ILVL  = ERROR RETURN CODE: =0 EXACT RESULT OBTAINED; =1 IF TOO   *
C*            MANY DATA ARE USED; <0 IF ERROR ENCOUNTERED              *
C*                                                                     *
C* SUBROUTINES NEEDED: KSSUM, DXP                                      *
C*                                                                     *
C* WRITTEN BY J. K. SHULTIS,  KANSAS STATE UNIVERSITY,  6/80           *
C*                                                                     *
C***********************************************************************
      REAL*8 G(MAXG),D,C(150),F(150),B(150),BLN(150),E(150),ELN(150)
      REAL*8 ARG,SUM,PMIN,PMAX,DXP,COMSUM
      EQUIVALENCE (C(1),F(1)), (B(1),E(1)), (BLN(1),ELN(1))
       COMMON/DTA/IDUMMY(502),IPRT,NITER
C
      NASY = 51
      KMAX = 149
      IDEBUG = 0
C
C*** CALCULATE WHETHER SMALL SAMPLE LEVEL OR ASYMPTOTIC LEVEL REQUIRED
      ILVL = 0
      IF (N.GE.NASY) GO TO 27
      STOP = N*(1.-D)
      NSTOP = STOP+1
      DELN = 1./N
      IF (NSTOP.GT.KMAX) GO TO 25
C
C*** CALCULATE THE C(K) COEFFICIENTS:
      J = 0
      K = 0
    1 IF (G(J+1).GE.(D+K*DELN)) GO TO 2
      J = J+1
      IF (J.GT.MAXG) GO TO 24
      GO TO 1
    2 C(K+1) = 1.-G(J+1)
      K = K+1
      IF (K-STOP) 1,3,4
    3 C(K+1) = 0.0
    4 IF (IDEBUG.GE.1) WRITE (IPRT,28) (C(I),I=1,NSTOP)
C FIND THE LAST NON-ZERO C(K):
      DO 5 K=1,NSTOP
      KK = NSTOP-K+1
      IF (C(KK).GT.0.0) GO TO 6
    5 CONTINUE
    6 MAXC = KK-1
      IF (MAXC.GT.0) GO TO 7
      PMIN = 0.0
      GO TO 11
C CALCULATE THE LOGS OF C(K):
    7 DO 8 K=1,KK
    8 C(K) = DLOG(C(K))
C
C CALCULATE THE B(K) COEFFICIENTS:
      BLN(1) = 0.0
      DO 9 KK=2,NSTOP
      K = KK-2
      KKK = KK-1
      JJ = MIN0(K,MAXC)
      CALL KSSUM (SUM,JJ,KKK,C,BLN,KMAX)
      COMSUM = 1.-SUM
      IF (COMSUM) 26,26,9
    9 BLN(KK) = DLOG(COMSUM)
      B(1) = 1.0
      DO 10 K=2,NSTOP
   10 B(K) = DXP(BLN(K))
      IF (IDEBUG.GE.1) WRITE (IPRT,29) (B(I),I=1,NSTOP)
C
C CALCULATE PROBABILITY(DMINUS > D)
      JJ = NSTOP-1
      JJ = MIN0(JJ,MAXC)
      CALL KSSUM (PMIN,JJ,N,C,BLN,KMAX)
   11 IF (IOUT.GE.1) WRITE (IPRT,30) PMIN
C
C CALCULATE THE F(K) COEFFICIENTS:
      K = 0
      J = 0
   12 IF (G(J+1).GT.(1.00001-D-K*DELN)) GO TO 13
      J = J+1
      IF (J.GT.MAXG) GO TO 24
      GO TO 12
   13 IF (J) 14,14,15
   14 F(K+1) = 0.0
      K = K+1
      IF (K-STOP) 14,14,16
   15 F(K+1) = G(J)
      J = J-1
      K = K+1
      IF (K-STOP) 12,12,16
   16 IF (IDEBUG.GE.1) WRITE (IPRT,31) (F(I),I=1,NSTOP)
C FIND THE LAST NON-ZERO F(I)
      DO 17 K=1,NSTOP
      KK = NSTOP-K+1
      IF (F(KK).GT.0.0) GO TO 18
   17 CONTINUE
   18 MAXF = KK-1
      IF (MAXF.GT.0) GO TO 19
      PMAX = 0.0
      GO TO 23
C CALCULATION OF THE LOGS OF F(K):
   19 DO 20 K=1,KK
   20 F(K) = DLOG(F(K))
C
C CALCULATE E(K) COEFFICIENTS:
      ELN(1) = 0.0
      DO 21 KK=2,NSTOP
      K = KK-2
      KKK = KK-1
      JJ = MIN0(K,MAXF)
      CALL KSSUM (SUM,JJ,KKK,F,ELN,KMAX)
      COMSUM = 1.-SUM
      IF (COMSUM) 26,26,21
   21 ELN(KK) = DLOG(COMSUM)
      E(1) = 1.0
      DO 22 K=2,NSTOP
   22 E(K) = DXP(ELN(K))
      IF (IDEBUG.GE.1) WRITE (IPRT,32) (E(I),I=1,NSTOP)
C
C CALCULATE PROB(DPLUS > D)
      JJ = NSTOP-1
      JJ = MIN0(JJ,MAXF)
      CALL KSSUM (PMAX,JJ,N,F,ELN,KMAX)
   23 IF (IOUT.GE.1) WRITE (IPRT,33) PMAX
C
C CALCULATE BOUNDS ON THE SIGNIFICANCE LEVEL
      ARG = PMIN*PMAX
      PMAX = PMAX+PMIN
      PMIN = PMAX-ARG
      IF (PMAX.GT.1.0) PMAX = 1.0
      RETURN
C
C
   24 WRITE (IPRT,34)
      ILVL = -1
      RETURN
   25 WRITE (IPRT,35)
      ILVL = -2
   26 WRITE (IPRT,36)
      ILVL = -3
      RETURN
C
C*** CALCULATE SIGNIFICANCE LEVEL FROM ASYMPTOTIC SERIES
   27 ILVL = 1
      PMAX = 1.0
      PMIN = 1.0
      WRITE (IPRT,37)
      RETURN
   28 FORMAT (//19H C(K) COEFFICIENTS:/(1H ,8G15.7))
   29 FORMAT (/19H B(I) COEFFICIENTS:/(1H ,8G15.7))
   30 FORMAT (/30H CALCULATED PROB(DMINUS > D) =,G15.7)
   31 FORMAT (/19H F(I) COEFFICIENTS:/(1H ,8G15.7))
   32 FORMAT (/19H E(I) COEFFICIENTS:/(1H ,8G15.7))
   33 FORMAT (/29H CALCULATED PROB(DPLUS > D) =,G15.7)
   34 FORMAT (32H TOO FEW G(K) SUPPLIED TO DKSLVL)
   35 FORMAT (46H NEED BIGGER ARRAYS IN DKSLVL FOR COEFFICIENTS)
   36 FORMAT (85H ROUND OFF ERROR PREVENTED ACCURATE CALCULATION OF BOUN
     1DS  FOR THE SIGNIFICANCE LEVEL)
   37 FORMAT (53H TOO MANY DATA TO PERFORM KOLMOGOROV-SMIRNOV FIT TEST)
      END
      SUBROUTINE KSSUM(SUM,JJ,N,ALN,BLN,MAX)
C***********************************************************************
C* THIS SUBROUTINE PERFORMS SUMMATIONS NEEDED IN SUBROUTINE DKSLVL     *
C***********************************************************************
      REAL*8 SUM,ALN(MAX),BLN(MAX),PLN,ARG,DXP
C
      PLN = 0.0
      SUM = DXP(N*ALN(1)+BLN(1))
      IF (JJ.EQ.0) RETURN
      JJJ = JJ+1
      DO 1 J=2,JJJ
      ARG = N-J+2
      PLN = PLN+DLOG(ARG/(J-1.))
    1 SUM = SUM+DXP(PLN+(N-J+1.)*ALN(J)+BLN(J))
      RETURN
      END
C
      REAL FUNCTION H*8(FF,TT)
C********************************** H **********************************
C*                                                                     *
C* SUBPROGRAM CALCULATES THE PROBABILITY OF OBTAINING FF FAILURES IN   *
C* TEST TIME TT WITH A HOMOGENEOUS POISSON MODEL (ITYPE = 0) OR WITH   *
C* A COMPOUND POISSON-GAMMA MODEL (ITYPE = 1).                         *
C*                                                                     *
C* INPUT VARIABLES:                                                    *
C*    FF   = NUMBER OF FAILURES                                        *
C*    TT   = COMPONENT TEST TIME                                       *
C*                                                                     *
C* COMMON BLOCK /MODEL/ A, B, ITYPE                                    *
C*    A     = MODEL PARAMETER: =LAMBDA FOR POISSON,                    *
C*                             =ALPHA FOR POISSON-GAMMA                *
C*    B     = MODEL PARAMETER: NOT USED FOR POISSON                    *
C*                             =BETA FOR POISSON-GAMMA                 *
C*    ITYPE = 0 FOR A HOMOGENEOUS POISSON MODEL; = 1 FOR THE COMPOUND  *
C*            POISSON-GAMMA MODEL                                      *
C*                                                                     *
C* SUBROUTINES NEEDED: DXP                                             *
C*                                                                     *
C***********************************************************************
C
      REAL*8 FF,TT,A,B,DXP,TAU,DLGAMA
      COMMON/MODEL/A,B,ITYPE
      IF (ITYPE) 1,1,2
C
C *** HOMOGENEOUS POISSON MODEL
    1 H = DXP(FF*DLOG(A*TT)-DLGAMA(FF+1.)-A*TT)
      RETURN
C
C*** COMPOUND POISSON-GAMMA MODEL
    2 TAU = 1./B
      H = DXP(DLGAMA(FF+A)-DLGAMA(FF+1.)-DLGAMA(A)+FF*DLOG(TT)-(FF+A)*DL
     1OG(TT+TAU)+A*DLOG(TAU))
      RETURN
      END
C
C
      REAL FUNCTION DXP*8(X)
C***********************************************************************
C*                                                                     *
C* THIS SUBROUTINE CALCULATES THE VALUE OF     DEXP(X)    FOR SMALL OR *
C* LARGE ARGUMENTS.                                                    *
C*                                                                     *
C***********************************************************************
      REAL*8 DEXP,X,DABS
      IF (DABS(X).GT.170.0D0) GO TO 1
      DXP = DEXP(X)
      RETURN
    1 IF (X) 2,2,3
    2 DXP = 0.0D0
      RETURN
    3 DXP = 1.0D70
      RETURN
      END
C
C
      SUBROUTINE GAMIC(X,ALPHA,REL,N,Y,NZ)
C***********************************************************************
C
C     WRITTEN BY D.E. AMOS AND S.L. DANIEL, NOVEMBER, 1974.
C     EXTRACTED FROM THE SANDIA LABORATORY'S MATHEMATICAL
C     LIBRARY AND ADAPTED FOR THE IBM 370 SYSTEM BY J. K. SHULTIS
C     JULY, 1979.
C
C     REFERENCE SC-DR-72 0303
C
C     ABSTRACT
C         GAMIC COMPUTES AN N MEMBER SEQUENCE OF INCOMPLETE GAMMA
C         FUNCTIONS NORMALIZED SO THAT AT X=INFINITY, THE INCOMPLETE
C         GAMMA FUNCTION HAS THE VALUE 1. THE SEQUENCE IS DENOTED BY
C
C         Y(K)=INCGAMMA(ALPHA+K-1,X)/GAMMA(ALPHA+K-1),  K=1,2,...,N
C
C         AND IS COMPUTED TO A RELATIVE ERROR REL OR BETTER WHERE ALPHA
C         .GT.0. IF ALPHA+N-1.GE.X, THE LAST MEMBER IS COMPUTED BY THE
C         CONFLUENT HYPERGEOMETRIC SERIES WITH THE OTHER MEMBERS
C         COMPUTED BY BACKWARD RECURSION ON A TWO-TERM FORMULA,
C
C           Y(K-1)=Y(K)+DEXP((ALPHA+K-1)*DLOG(X)-X-DLGAMA(ALPHA+K)).
C
C         IF ALPHA+N-1.LT.X, AN INTEGER M IS ADDED SO THAT
C         ALPHA+N-1+M.GE.X AND THE FIRST PROCEDURE IS APPLIED. SPECIAL
C         PROCEDURES APPLY FOR ALPHA.EQ.1 OR AN UNDERFLOW OCCURS OR
C         X EXCEEDS A CRITICAL VALUE, APTEST, WHERE ALL MEMBERS ARE 1.
C         TO THE WORD LENGTH OF THE CDC 6600. GAMIC USES DLGAMA.
C
C     DESCRIPTION OF ARGUMENTS
C
C         INPUT
C           X      - ARGUMENT, X.GE.0.0
C           ALPHA  - PARAMETER, ALPHA.GT.0.0
C           REL    - RELATIVE ERROR TOLERANCE, REL=1.E-S FOR S
C                    SIGNIFICANT DIGITS
C           N      - NUMBER OF GAMMA FUNCTIONS IN THE SEQUENCE
C                    BEGINNING AT PARAMETER ALPHA, N.GE.1
C
C         OUTPUT
C           Y      - A VECTOR CONTAINING AN N MEMBER SEQUENCE
C                    Y(K)= INCGAMMA(ALPHA+K-1,X)/GAMMA(ALPHA+K-1),
C                    K=1,...,N TO A RELATIVE ERROR REL.
C           NZ     - UNDERFLOW FLAG
C                    NZ.EQ.0, A NORMAL RETURN
C                    NZ.NE.0, UNDERFLOW, Y(K)=0.0, K=N-NZ+1,N RETURNED
C
C     ERROR CONDITIONS
C         IMPROPER INPUT PARAMETERS - A FATAL ERROR
C         UNDERFLOW - A NON-FATAL ERROR.
C
C***********************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 X ALPHA,REL,Y,DLOG,DEXP,DFLOAT
      DIMENSION Y(1)
      DIMENSION AA(6),BB(6),CC(5)
C
      DATA   AA            / 1.18399941922176, 330.888136276861,
     1 10490.832947926, 37842.0325596908, 157.586618187374,
     2 1305.69632410551/
      DATA   BB            / 1.02652821626751, 9297.53107520368,
     1 6538489.23630220, 289543295.992889, 8168.36456953161,
     2 4122376.56364399/
      DATA   CC            / 430952.856710482, 8279882567.43362,
     1 2419444686844.45, 421722.873236008, 7565938027.47116/
      DATA SLOG/-160.D0/
      DATA  CON14          / 33.2361913019165/
      DATA  SCALE          /1.D-60/
      DATA  ASCL           /1.D-16/
C
      IF (REL.LE.0.) GO TO 22
      IF (N.LT.1) GO TO 23
      IF (ALPHA.LE.0.0) GO TO 24
      NZ = 0
      IF (X) 25,1,3
    1 DO 2 I=1,N
    2 Y(I) = 0.
      RETURN
C
C     IF X.GE.XLIM(ALPHA+N-1), THEN Y(K)=1., K=1,2,...,N
C
    3 NN = N
      RX = 1./X
      NBAR = 0
      APN = ALPHA+DFLOAT(N)-1.
      APTEST = APN
      AM1 = ALPHA-1.
      IF (APN.LE.1.) GO TO 10
    4 IF (X.LE.APTEST) GO TO 13
      IF (APTEST.GT.200.) GO TO 5
      S1 = ((AA(1)*APTEST+AA(2))*APTEST+AA(3))*APTEST+AA(4)
      S2 = (APTEST+AA(5))*APTEST+AA(6)
      GO TO 6
    5 IF (APTEST.GT.10000.) GO TO 7
      S1 = ((BB(1)*APTEST+BB(2))*APTEST+BB(3))*APTEST+BB(4)
      S2 = (APTEST+BB(5))*APTEST+BB(6)
    6 XLIM = S1/S2
C
      IF (X.GE.XLIM) GO TO 8
      GO TO 12
    7 S1 = ((APTEST+CC(1))*APTEST+CC(2))*APTEST+CC(3)
      S2 = (APTEST+CC(4))*APTEST+CC(5)
      GO TO 6
    8 DO 9 I=1,N
    9 Y(I) = 1.
      RETURN
C
   10 IF (ALPHA.NE.1.) GO TO 11
      IF (X.GT.CON14) GO TO 8
      IF (X.LT.0.1) GO TO 13
      Y(NN) = 1.-DEXP(-X)
      RETURN
   11 NBAR = 1
      APTEST = APN+1.
      GO TO 4
   12 NBAR = X-APTEST+5.+DFLOAT(NBAR)
   13 ABAR = APN+DFLOAT(NBAR)
      XLOG = DLOG(X)
      A1 = 1.
      SUM = 1.
      ABK = ABAR+1.
   14 A1 = A1*X/ABK
      SUM = SUM+A1
      IF (A1.LT.REL) GO TO 15
      ABK = ABK+1.
      GO TO 14
   15 YY = SUM*SCALE
      D = ABAR
      IF (NBAR.EQ.0) GO TO 18
   16 CONTINUE
      DO 17 K=1,NBAR
      XOD = X/D
      IF (XOD.LT.ASCL) GO TO 17
      YY = XOD*YY+SCALE
   17 D = D-1.
C
      IF (NZ.NE.0) GO TO 19
   18 E = -X+D*XLOG-DLGAMA(D+1.)
   19 IF (E.GE.SLOG) GO TO 20
      Y(NN) = 0.
      NZ = NZ+1
      NN = NN-1
      IF (NN.EQ.0) RETURN
      NBAR = 1
      APN = APN-1.
      E = E+DLOG(D*RX)
      GO TO 16
   20 EXE = DEXP(E)
      Y(NN) = (EXE/SCALE)*YY
      NM1 = NN-1
      IF (NM1.EQ.0) RETURN
      F = EXE*APN*RX
      KK = NN
      AK = DFLOAT(NN)+AM1
      DO 21 K=1,NM1
      Y(KK-1) = Y(KK)+F
      KK = KK-1
      AK = AK-1.
      F = F*AK*RX
   21 CONTINUE
      RETURN
C
   22 WRITE (9,26)
      RETURN
   23 WRITE (9,27)
      RETURN
   24 WRITE (9,28)
      RETURN
   25 WRITE (9,29)
      RETURN
   26 FORMAT (33HIN GAMIC, IMPROPER INPUT FOR REL.)
   27 FORMAT (31HIN GAMIC, IMPROPER INPUT FOR N.)
   28 FORMAT (35HIN GAMIC, IMPROPER INPUT FOR ALPHA.)
   29 FORMAT (31HIN GAMIC, IMPROPER INPUT FOR X.)
      END
C
C
C
      REAL FUNCTION DLGAMA*8(XX)
      REAL*8 COF(6),STP,HALF,ONE,FPF,X,TMP,SER,XX
      DATA COF,STP/76.18009173D0,-86.50532033D0,24.01409822D0,
     *    -1.231739516D0,.120858003D-2,-.536382D-5,2.50662827465D0/
      DATA HALF,ONE,FPF/0.5D0,1.0D0,5.5D0/
      X=XX-ONE
      TMP=X+FPF
      TMP=(X+HALF)*LOG(TMP)-TMP
      SER=ONE
      DO 11 J=1,6
        X=X+ONE
        SER=SER+COF(J)/X
11    CONTINUE
      DLGAMA=TMP+LOG(STP*SER)
      RETURN
      END
